/*
 * Copyright 2008 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define _CONDOR_ALLOW_OPEN
#include "condor_common.h"

#include "JobUtils.h"

#include "condor_classad.h"

#include "condor_qmgr.h"


enum AttributeType {
	EXPR_TYPE = 0,
	INTEGER_TYPE = 1,
	FLOAT_TYPE = 2,
	STRING_TYPE = 3
};


bool
PopulateFieldTableFromJobAd(int clusterId,
							int procId,
							qpid::framing::FieldTable &fieldTable)
{
	static const char *TYPE_ATTR = "TYPE";
	static const char *VALUE_ATTR = "VALUE";

	ExprTree *expr;
	char *name;
	char *value;
	ClassAd *ad;
	qpid::framing::FieldTable attr;

	if (NULL == (ad = GetJobAd(clusterId, procId, false))) {
		dprintf(D_ALWAYS,
				"GetAd method called on %d.%d, which does not exist\n",
				clusterId, procId);
		return false;
	}

	ad->ResetExpr();
	while (NULL != (expr = ad->NextExpr())) {
		attr = qpid::framing::FieldTable();
		name = ((Variable *) expr->LArg())->Name();

			// All these extra lookups are horrible. They have to
			// be there because the ClassAd may have multiple
			// copies of the same attribute name! This means that
			// the last attribute with a given name will set the
			// value, but the last attribute tends to be the
			// attribute with the oldest (wrong) value. How
			// annoying is that!
		if (!(expr = ad->Lookup(name))) {
			dprintf(D_ALWAYS, "Failed to lookup %s\n", name);

			return false;
		}

		switch (expr->RArg()->MyType()) {
		case LX_INTEGER:
			attr.setInt(TYPE_ATTR, INTEGER_TYPE);
			break;
		case LX_FLOAT:
			attr.setInt(TYPE_ATTR, FLOAT_TYPE);
			break;
		case LX_STRING:
			attr.setInt(TYPE_ATTR, STRING_TYPE);
			break;
		default:
			attr.setInt(TYPE_ATTR, EXPR_TYPE);
		}

		expr->RArg()->PrintToNewStr(&value);
		attr.setString(VALUE_ATTR, value);
		free(value);

		fieldTable.setTable(name, attr);
	}

	return true;
}
