/*
 * Copyright 2008 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define _CONDOR_ALLOW_OPEN
#include "condor_common.h"

#include "../condor_collector.V6/CollectorPlugin.h"
#include "condor_classad.h"

#include "condor_commands.h"
#include "command_strings.h"
#include "../condor_collector.V6/hashkey.h"

#include "../condor_daemon_core.V6/condor_daemon_core.h"

#include "condor_attributes.h"

#include "condor_config.h"

#include "qpid/management/Manageable.h"
#include "qpid/management/ManagementObject.h"
#include "qpid/agent/ManagementAgent.h"
//#include "qpid/framing/FieldTable.h"

#include "Slot.h"
#include "Collector.h"

#include "PoolUtils.h"

#include "MgmtConversionMacros.h"

extern DaemonCore *daemonCore;

extern int main_shutdown_graceful();

using namespace std;
using namespace qpid::management;
using namespace qpid::framing;
using namespace qmf::mrg::grid;

struct CollectorObject : public Manageable
{
	Collector *mgmtObject;

	CollectorObject(ManagementAgent *agent)
	{
		mgmtObject = new Collector(agent, this);

			// The Collector is always object 1 in its agent
		agent->addObject(mgmtObject, 1);

		advertise();
	}

	~CollectorObject() { };

	ManagementObject *GetManagementObject(void) const
	{
		return mgmtObject;
	}

private:

	void advertise()
	{
		ClassAd ad;
		MGMT_DECLARATIONS;

			// XXX: This will look very familiar to code in void
			// CollectorDaemon::init_classad(int interval), sigh.

		char *CollectorName = param("COLLECTOR_NAME");

		ad.SetMyTypeName(COLLECTOR_ADTYPE);
		ad.SetTargetTypeName("");

		char *tmp;
		tmp = param( "CONDOR_ADMIN" );
		if( tmp ) {
			ad.Assign( ATTR_CONDOR_ADMIN, tmp );
			free( tmp );
		}

		if( CollectorName ) {
			ad.Assign( ATTR_NAME, CollectorName );
		} else {
			ad.Assign( ATTR_NAME, my_full_hostname() );
		}

		ad.Assign( ATTR_COLLECTOR_IP_ADDR, global_dc_sinful() );

		daemonCore->publish(&ad);

		mgmtObject->set_Pool(GetPoolName());
		mgmtObject->set_System(my_full_hostname());

		STRING(CondorPlatform);
		STRING(CondorVersion);
		STRING(Name);
		STRING(PublicNetworkIpAddr);
	}
};

struct SlotObject : public Manageable
{
	Slot *mgmtObject;

	SlotObject(ManagementAgent *agent)
	{
		mgmtObject = new Slot(agent, this);

		agent->addObject(mgmtObject);
	}

	~SlotObject() { };

	ManagementObject *GetManagementObject(void) const
	{
		return mgmtObject;
	}

	void update(const ClassAd &ad)
	{
		MGMT_DECLARATIONS;
//		string name;

			// Logic to set values on mgmtObject

//		FieldTable fieldTable;
//
//		((ClassAd) ad).ResetExpr();
//		while (NULL != (expr = ((ClassAd) ad).NextExpr())) {
//			name = ((Variable *) expr->LArg())->Name();
// 
//			switch (expr->RArg()->MyType()) {
//			case LX_INTEGER:
//				num = (uint32_t) ((Integer *) expr->RArg())->Value();
//				fieldTable.setInt(name, num);
//				break;
//			case LX_FLOAT:
//				dbl = (double) ((Float *) expr->RArg())->Value();
//				fieldTable.setInt(name, (int) dbl);
//				break;
//			case LX_STRING:
//				str = ((String *) expr->RArg())->Value();
//				fieldTable.setString(name, str);
//				break;
//			default:
//				expr->RArg()->PrintToNewStr(&str);
//				fieldTable.setString(name, str);
//				free(str);
//			}
//		}

		mgmtObject->set_Pool(GetPoolName());

			// There is no CondorPlatform or CondorVersion ATTR_*
			// definition
		STRING(CondorPlatform);
		STRING(CondorVersion);

		TIME_INTEGER(DaemonStartTime);
		STRING(Arch);
		STRING(CheckpointPlatform);
		INTEGER(Cpus);
		INTEGER(Disk);
		STRING(FileSystemDomain);
		EXPR(IsValidCheckpointPlatform);
		INTEGER(KFlops);
		STRING(Machine);
		EXPR(MaxJobRetirementTime);
		INTEGER(Memory);
		INTEGER(Mips);
		STRING(MyAddress);
		STRING(Name);
		STRING(OpSys);
		EXPR(Requirements);
		STRING(PublicNetworkIpAddr);
		EXPR(Rank);
		INTEGER(SlotID);
		EXPR(Start);
		STRING(StarterAbilityList);
		INTEGER(TotalCpus);
		INTEGER(TotalDisk);
		INTEGER(TotalMemory);
		INTEGER(TotalSlots);
		INTEGER(TotalVirtualMemory);
		STRING(UidDomain);
		INTEGER(VirtualMemory);
		INTEGER(WindowsBuildNumber);
		INTEGER(WindowsMajorVersion);
		INTEGER(WindowsMinorVersion);

			// The MonitorSelf* attributes do not have ATTR_*
			// definitions
		INTEGER(MonitorSelfAge);
		DOUBLE(MonitorSelfCPUUsage);
		DOUBLE(MonitorSelfImageSize);
		INTEGER(MonitorSelfRegisteredSocketCount);
		INTEGER(MonitorSelfResidentSetSize);

		OPT_STRING(AccountingGroup);
		STRING(Activity);
		OPT_STRING(ClientMachine);
		INTEGER(ClockDay);
		INTEGER(ClockMin);
		OPT_STRING(ConcurrencyLimits);
		DOUBLE(CondorLoadAvg);
		INTEGER(ConsoleIdle);
		OPT_DOUBLE(CurrentRank);
		TIME_INTEGER(EnteredCurrentActivity);
		TIME_INTEGER(EnteredCurrentState);
		OPT_STRING(GlobalJobId);
		OPT_INTEGER(ImageSize);
		OPT_STRING(JobId);
		OPT_TIME_INTEGER(JobStart);
		INTEGER(KeyboardIdle);
		TIME_INTEGER(LastBenchmark);
		TIME_INTEGER(LastFetchWorkCompleted);
		TIME_INTEGER(LastFetchWorkSpawned);
		TIME_INTEGER(LastPeriodicCheckpoint);
		DOUBLE(LoadAvg);
		TIME_INTEGER(MyCurrentTime);
		INTEGER(NextFetchWorkDelay);
		OPT_STRING(PreemptingConcurrencyLimits);
		OPT_STRING(PreemptingOwner);
		OPT_STRING(PreemptingUser);
		OPT_DOUBLE(PreemptingRank);
		OPT_STRING(RemoteOwner);
		OPT_STRING(RemoteUser);
		STRING(State);
		INTEGER(TimeToLive);
		OPT_INTEGER(TotalClaimRunTime);
		OPT_INTEGER(TotalClaimSuspendTime);
		DOUBLE(TotalCondorLoadAvg);
		OPT_INTEGER(TotalJobRunTime);
		OPT_INTEGER(TotalJobSuspendTime);
		DOUBLE(TotalLoadAvg);
		INTEGER(TotalTimeBackfillBusy);
		INTEGER(TotalTimeBackfillIdle);
		INTEGER(TotalTimeBackfillKilling);
		INTEGER(TotalTimeClaimedBusy);
		INTEGER(TotalTimeClaimedIdle);
		INTEGER(TotalTimeClaimedRetiring);
		INTEGER(TotalTimeClaimedSuspended);
		INTEGER(TotalTimeMatchedIdle);
		INTEGER(TotalTimeOwnerIdle);
		INTEGER(TotalTimePreemptingKilling);
		INTEGER(TotalTimePreemptingVacating);
		INTEGER(TotalTimeUnclaimedBenchmarking);
		INTEGER(TotalTimeUnclaimedIdle);

			// XXX: While we do not have the System's UUID we will use
			// its name. Right now this is a placeholder so when we do
			// get a UUID users of the System attribute don't have
			// to change.
		mgmtObject->set_System(mgmtObject->get_Machine());
	}
};


struct MgmtCollectorPlugin : public Service, CollectorPlugin
{
		// ManagementAgent::Singleton cleans up the ManagementAgent
		// instance if there are no ManagementAgent::Singleton's in
		// scope!
	ManagementAgent::Singleton *singleton;

	typedef HashTable<AdNameHashKey, SlotObject *> SlotHashTable;

	SlotHashTable *startdAds;

	CollectorObject *collector;

	void
	initialize()
	{
		char *host;
		int port;
		char *tmp;
		string storefile;

		dprintf(D_FULLDEBUG, "MgmtCollectorPlugin: Initializing...\n");

		singleton = new ManagementAgent::Singleton();

		startdAds = new SlotHashTable(4096, &adNameHashFunction);

		ManagementAgent *agent = singleton->getInstance();

		Slot::registerSelf(agent);
		Collector::registerSelf(agent);

		port = param_integer("QMF_BROKER_PORT", 5672);
		if (NULL == (host = param("QMF_BROKER_HOST"))) {
			host = strdup("localhost");
		}

		tmp = param("QMF_STOREFILE");
		if (NULL == tmp) {
			storefile = ".collector_storefile";
		} else {
			storefile = tmp;
			free(tmp); tmp = NULL;
		}
		
		agent->init(string(host), port,
					param_integer("QMF_UPDATE_INTERVAL", 10),
					true,
					storefile);

		free(host);

		collector = new CollectorObject(agent);

/*
		ReliSock *sock = new ReliSock;
		if (!sock) {
			EXCEPT("Failed to allocate Mgmt socket");
		}
		if (!sock->assign(agent->getSignalFd())) {
			EXCEPT("Failed to bind Mgmt socket");
		}
		int index;
		if (-1 == (index =
				   daemonCore->Register_Socket((Stream *) sock,
											   "Mgmt Method Socket",
											   (SocketHandlercpp)
											   &MgmtCollectorPlugin::HandleMgmtSocket,
											   "Handler for Mgmt Methods.",
											   this))) {
			EXCEPT("Failed to register Mgmt socket");
		}
*/

			//printf("Making Ted Happy!\n");
	}

	void
	shutdown()
	{
		if (!param_boolean("QMF_DELETE_ON_SHUTDOWN", false)) {
			return;
		}

		dprintf(D_FULLDEBUG, "MgmtCollectorPlugin: shutting down...\n");
		if (singleton) {
			delete singleton;
			singleton = NULL;
		}
	}

	void
	update(int command, const ClassAd &ad)
	{
		AdNameHashKey hashKey;
		SlotObject *slotObject;

		switch (command) {
		case UPDATE_STARTD_AD:
			dprintf(D_FULLDEBUG, "MgmtCollectorPlugin: Received UPDATE_STARTD_AD\n");
			if (!makeStartdAdHashKey(hashKey, ((ClassAd *) &ad), NULL)) {
				dprintf(D_FULLDEBUG, "Could not make hashkey -- ignoring ad\n");
			}

			if (startdAds->lookup(hashKey, slotObject)) {
					// Key doesn't exist
				slotObject = new SlotObject(singleton->getInstance());

					// Ignore old value, if it existed (returned)
				startdAds->insert(hashKey, slotObject);
			}

			slotObject->update(ad);

			break;
		default:
			dprintf(D_FULLDEBUG, "MgmtCollectorPlugin: Unsupported command: %s\n",
					getCollectorCommandString(command));
		}
	}

	void
	invalidate(int /*command*/, const ClassAd &/*ad*/)
	{
			//printf("Invalidate\n");
	}


	int
	HandleMgmtSocket(Service *, Stream *)
	{
		singleton->getInstance()->pollCallbacks();

		return KEEP_STREAM;
	}
};

static MgmtCollectorPlugin instance;
